# CSS Scoping Module Level 1

## Editor’s Draft, 1 January 1970

Specification Metadata

This version:
  https://drafts.csswg.org/css-scoping/
Latest published version:
  https://www.w3.org/TR/css-scoping-1/
Previous Versions:
  https://www.w3.org/TR/2014/WD-css-scoping-1-20140403/
Test Suite:
  http://test.csswg.org/suites/css-scoping-1_dev/nightly-unstable/
Editor:
  Tab Atkins Jr. (Google)
Issue Tracking:
  GitHub Issues
Suggest an Edit for this Spec:
  GitHub Editor

Copyright © 1970 W3C® (MIT, ERCIM, Keio, Beihang). W3C liability, trademark and permissive document license rules apply.

## Abstract

This specification defines scoping/encapsulation mechanisms for CSS, focusing on the Shadow DOM scoping mechanism.

CSS is a language for describing the rendering of structured documents (such as HTML and XML) on screen, on paper, etc.

## Status of this document

This is a public copy of the editors’ draft. It is provided for discussion only and may change at any moment. Its publication here does not imply endorsement of its contents by W3C. Don’t cite this document other than as work in progress.

GitHub Issues are preferred for discussion of this specification. When filing an issue, please put the text “css-scoping” in the title, preferably like this: “[css-scoping] …summary of comment…”. All issues and comments are archived, and there is also a historical archive.

This document was produced by the CSS Working Group.

This document was produced by a group operating under the W3C Patent Policy. W3C maintains a public list of any patent disclosures made in connection with the deliverables of the group; that page also includes instructions for disclosing a patent. An individual who has actual knowledge of a patent which the individual believes contains Essential Claim(s) must disclose the information in accordance with section 6 of the W3C Patent Policy.

This document is governed by the 1 March 2019 W3C Process Document.

## 1. Introduction

...

## 2. Default Styles for Custom Elements

When defining custom elements, one often wants to set up "default" styles for them, akin to the user-agent styles that apply to built-in elements. This is, unfortunately, hard to do in vanilla CSS, due to issues of scoping and specificity—the element in question might be used in shadow trees, and thus is unreachable by any selector targeting it in the outermost document; and selectors, even low-specificity ones like simple type selectors, can accidentally override author-level styles meant to target the element.

To aid in this, this section defines a way to create a stylesheet of "default element styles" for a given element. This stylesheet applies across the entire document, in all shadow trees, and the rules in it apply at the user agent origin, so author-level rules automatically win.

`Window`s gain a private slot `[[defaultElementStylesMap]]` which is a map of local names to stylesheets.

These stylesheets must apply to every document in the window. They must be interpreted as user agent stylesheets.

Note: This implies, in particular, that they apply to all shadow trees in every document, and that the declarations in them are from the user agent origin.

For the purpose of the cascade, these stylesheets are ordered after the user agent’s own stylesheets; their relative ordering doesn’t matter as it is not observable.

Within these stylesheets, complex selectors must be treated as invalid. Every compound selector must be treated as containing an additional type selector that selects elements with the local name that the stylesheet is keyed with.

Do we need to restrict the at-rules that can be used in these sheets? For example, do we allow an @font-face? I’m going to leave it as allowed unless/until I hear complaints.

This specification does not define how to add to, remove from, or generally manipulate the `[[defaultElementStylesMap]]`. It is expected that other specifications, such as [DOM], will define ways to do so.

## 3. Shadow Encapsulation

### 3.1. Informative Explanation of Shadow DOM

The following is a non-normative explanation of several concepts normatively defined in the DOM Standard [DOM], to aid in understanding what this spec defines without having to fully grok the DOM Standard.

In addition to the qualities of an element tree defined in Selectors Level 4 §data-model, the DOM Standard adds several new concepts related to shadow trees, several of which are relevant to CSS.

An element can host a shadow tree, which is a special kind of document fragment with a shadow root (a non-element node) at its root. Children of the shadow root are ordinary elements and other nodes. The element hosting the shadow tree is its host, or shadow host.

The elements in a shadow tree are not descendants of the shadow host in general (including for the purposes of Selectors like the descendant combinator). However, the shadow tree, when it exists, is used in the construction of the flattened element tree, which CSS uses for all purposes after Selectors (including inheritance and box construction).

Loosely, the shadow tree is treated as the shadow host’s contents instead of its normal light tree contents. However, some of its light tree children can be "pulled into" the shadow tree by assigning them to slots. This causes them to be treated as children of the slot for CSS purposes. The slots can then be assigned to slots in deeper shadow trees; luckily, slots themselves don’t generate boxes by default, so you don’t get an unpredictable cascade of `slot` wrapper elements disrupting your CSS.

If nothing is explicitly assigned to a slot, the slot’s own children are instead assigned to it, as a sort of "default" contents.

### 3.2. Shadow DOM and Selectors

#### 3.2.1. Matching Selectors Against Shadow Trees

When a selector is matched against a shadow tree, the selector match list is initially the shadow host, followed by all children of the shadow tree’s shadow root and their descendants, ordered by a pre-order traversal.

Note: Remember that the descendants of an element are based on the light tree children of the element, which does not include the shadow trees of the element.

When a selector is matched against a tree, its tree context is the root of the root elements passed to the algorithm. If the tree context is a shadow root, that selector is being matched in the context of a shadow tree.

For example, any selector in a stylesheet embedded in or linked from an an element in a shadow tree is in the context of a shadow tree. So is the argument to `querySelector()` when called from a shadow root.

Declarations inherit the tree context of the selector that was matched to apply them.

#### 3.2.2. Selecting Shadow Hosts from within a Shadow Tree

A shadow host is outside of the shadow tree it hosts, and so would ordinarily be untargettable by any selectors evaluated in the context of the shadow tree (as selectors are limited to a single tree), but it is sometimes useful to be able to style it from inside the shadow tree context.

For the purpose of Selectors, a shadow host also appears in its shadow tree, with the contents of the shadow tree treated as its children. (In other words, the shadow host is treated as replacing the shadow root node.)

When considered within its own shadow trees, the shadow host is featureless. Only the :host, :host(), and :host-context() pseudo-classes are allowed to match it.

Why is the shadow host so weird?

The shadow host lives outside the shadow tree, and its markup is in control of the page author, not the component author.

It would not be very good if a component used a particular class name internally in a shadow tree stylesheet, and the page author using the component accidentally also used the the same class name and put it on the shadow host. Such a situation would result in accidental styling that is impossible for the component author to predict, and confusing for the page author to debug.

However, there are still some reasonable use-cases for letting a stylesheet in a shadow tree style its shadow host. (For example, the component might want to be laid out as a flexbox, requiring the shadow host to be set to display: flex.) So, to allow this situation but prevent accidental styling, the shadow host appears but is completely featureless and unselectable except through :host and its related functional forms, which make it very explicit when you’re trying to match against markup provided by the page author.

#### 3.2.3. Selecting Into the Light: the :host, :host(), and :host-context() pseudo-classes

The :host pseudo-class, when evaluated in the context of a shadow tree, matches the shadow tree’s shadow host. In any other context, it matches nothing.

The :host() function pseudo-class has the syntax:

    :host( <compound-selector-list> )

When evaluated in the context of a shadow tree, it matches the shadow tree’s shadow host if the shadow host, in its normal context, matches the selector argument. In any other context, it matches nothing.

The specificity of :host is that of a pseudo-class. The specificity of :host() is that of a pseudo-class, plus the specificity of its argument.

Note: This is different from the specificity of similar pseudo-classes, like :matches() or :not(), which only take the specificity of their argument. This is because :host is affirmatively selecting an element all by itself, like a "normal" pseudo-class; it takes a selector argument for syntactic reasons (we can’t say that :host.foo matches but .foo doesn’t), but is otherwise identical to just using :host followed by a selector.

For example, say you had a component with a shadow tree like the following:

    <x-foo class="foo">
      <"shadow tree">
        <div class="foo">...</div>
      </>
    </x-foo>

For a stylesheet within the shadow tree:

- :host matches the `<x-foo>` element.

- x-foo matches nothing.

- .foo matches only the `<div>` element.

- .foo:host matches nothing

- :host(.foo) matches the `<x-foo>` element.

Ordinary, selectors within a shadow tree can’t see elements outside the shadow tree at all. Sometimes, however, it’s useful to select an ancestor that lies somewhere outside the shadow tree, above it in the document.

For example, a group of components can define a handful of color themes they they know how to respond to. Page authors could opt into a particular theme by adding a specific class to the components, or higher up in the document.

The :host-context() functional pseudo-class tests whether there is an ancestor, outside the shadow tree, which matches a particular selector. Its syntax is:

    :host-context( <compound-selector-list> )

When evaluated in the context of a shadow tree, the :host-context() pseudo-class matches the shadow host, if the shadow host or one of its shadow-including ancestors matches the provided <compound-selector-list>. In any other context, it matches nothing.

The specificity of :host-context() is that of a pseudo-class, plus the specificity of its argument.

Note: This means that the selector pierces through shadow boundaries on the way up, looking for elements that match its argument, until it reaches the document root.

#### 3.2.4. Selecting Slot-Assigned Content: the ::slotted() pseudo-element

The ::slotted() pseudo-element represents the elements assigned, after flattening, to a slot. This pseudo-element only exists on slots.

The ::slotted() pseudo-element is an alias for other elements in the tree, and does not generate any boxes itself.

The grammar of the ::slotted() pseudo-element is:

    ::slotted( <compound-selector-list> )

The ::slotted() pseudo-element represents the elements that are:

- assigned, after flattening, to the slot that is ::slotted’s originating element

- matched by its <compound-selector-list> argument

The ::slotted() pseudo-element can be followed by a tree-abiding pseudo-element, like ::slotted()::before, representing the appropriate pseudo-element of the elements represented by the ::slotted() pseudo-element.

The specificity of ::slotted() is that of a pseudo-element, plus the specificity of its argument.

For example, say you had a component with both children and a shadow tree, like the following:

    <x-foo>
      <div id="one" slot="foo" class="foo">...</div>
      <div id="two" slot="foo">...</div>
      <div id="three" class="foo">
        <div id="four" slot="foo">...</div>
      </div>
      <"shadow tree">
        <div id="five">...</div>
        <div id="six">...</div>
        <slot name="foo"></slot>
      </"shadow tree">
    </x-foo>

For a stylesheet within the shadow tree, a selector like ::slotted(*) selects #one and #two only, as they’re the elements assigned to the sole `slot` element. It will not select #three (no `slot` attribute) nor #four (only direct children of a shadow host can be assigned to a slot).

A selector like ::slotted(.foo), on the other hand, will only select #one, as it matches .foo, but #two doesn’t.

Note: Note that a selector like ::slotted(*) is equivalent to *::slotted(*), where the * selects many more elements than just the `slot` element. However, since only the `slot` elements are slots, they’re the only elements with a ::slotted() pseudo-element as well.

Note: ::slotted() can only represent the elements assigned to the slot. Slots can also be assigned text nodes, which can’t be selected by ::slotted(). The only way to style assigned text nodes is by styling the slot and relying on inheritance.

### 3.3. Shadow Trees and the Cascade

To address the desired cascading behavior of rules targetting elements in shadow roots, this specification extends the cascade order defined in the Cascade specification. [CSS3CASCADE]

An additional cascade criteria must be added, between Origin and Scope, called Shadow Tree.

- When comparing two declarations that have different tree contexts, then for normal rules the declaration earlier in the shadow-including tree order wins, and for important rules the declaration coming later in the shadow-including tree order wins.

  Note: This is the opposite of how scoped styles work.

### 3.4. Flattening the DOM into an Element Tree

While Selectors operates on the DOM tree as the host language presents it, with separate trees that are unreachable via the standard parent/child relationship, the rest of CSS needs a single unified tree structure to work with. This is called the flattened element tree (or flat tree), and is constructed as follows:

1. Let pending nodes be a list of DOM nodes with associated parents, initially containing just the document’s root element with no associated parent.

2. Repeatedly execute the following substeps until pending nodes is empty:

   1. Pop the first element from pending nodes, and assign it to pending node.

   2. Insert pending node into the flat tree as a child of its associated parent. (If it has no associated parent, it’s the document root—just insert it into the flat tree as its root.)

   3. Perform one of the following, whichever is the first that matches:

      pending node is a shadow host
        Append the child nodes of the shadow root of the shadow tree it hosts to pending nodes, with pending node as their associated parent.
      pending node is a slot
        Find slotables for pending node, and append them to pending nodes, with pending node as their associated parent.

        If no slotables were found for pending node, instead append its children to pending nodes, with pending node as their associated parent.

      Otherwise,
        Append the child nodes of pending node’s light tree to pending nodes, with pending node as their associated parent.

Note: In other words, the flat tree is the top-level DOM tree, but shadow hosts are filled with their shadow tree children instead of their light tree children (and this proceeds recursively if the shadow tree contains any shadow hosts), and slots get filled with the nodes that are assigned to them (and this proceeds recursively if the slots are themselves assigned to a slot in a deeper shadow tree).

A non-obvious result of this is that elements assigned to a slot inherit from that slot, not their light-tree parent or any deeper slots their slot gets assigned to. This means that text nodes are styled by the shadow tree of their parent, with nobody else capable of intervening in any way. Do we want an additional pseudo-element for targeting those text nodes so they can be styled at all slot-assignment levels, like normal elements can be? This implies it needs to work for text nodes in the light tree before they’re assigned downwards, so this can’t just be a ::slotted() variant. Luckily, this is a long-standing request!

#### 3.4.1. Slots and Slotted Elements in a Shadow Tree

Slots must act as if they were assigned display: contents via a rule in the UA origin. This must be possible to override via display, so they do generate boxes if desired.

Note: A non-obvious result of assigning elements to slots is that they inherit from the slot they’re assigned to. Their original light tree parent, and any deeper slots that their slot gets assigned to, don’t affect inheritance.

## 4. Changes

The following significant changes were made since the 3 April 2014 Working Draft.

- Renamed ::content to ::slotted.

- Define the flattened tree

- Generally reorg and rebase the Shadow DOM section on top of current DOM.

- Punt @scope and related things, and ::region and related things, to the next level of the draft.

## 5. Privacy and Security Considerations

This specification introduces Shadow DOM and some shadow-piercing capabilities, but this does not introduce any privacy or security issues—shadow DOM, as currently specified, is intentionally not a privacy/security boundary (and the parts of the UA that use shadow DOM and do have a privacy/security boundary implicitly rely on protections not yet specified, which protect them from the things defined in this specification).

## Conformance

### Document conventions

Conformance requirements are expressed with a combination of descriptive assertions and RFC 2119 terminology. The key words “MUST”, “MUST NOT”, “REQUIRED”, “SHALL”, “SHALL NOT”, “SHOULD”, “SHOULD NOT”, “RECOMMENDED”, “MAY”, and “OPTIONAL” in the normative parts of this document are to be interpreted as described in RFC 2119. However, for readability, these words do not appear in all uppercase letters in this specification.

All of the text of this specification is normative except sections explicitly marked as non-normative, examples, and notes. [RFC2119]

Examples in this specification are introduced with the words “for example” or are set apart from the normative text with `class="example"`, like this:

This is an example of an informative example.

Informative notes begin with the word “Note” and are set apart from the normative text with `class="note"`, like this:

Note, this is an informative note.

Advisements are normative sections styled to evoke special attention and are set apart from other normative text with `<strong class="advisement">`, like this: UAs MUST provide an accessible alternative.

### Conformance classes

Conformance to this specification is defined for three conformance classes:

style sheet
  A CSS style sheet.
renderer
  A UA that interprets the semantics of a style sheet and renders documents that use them.
authoring tool
  A UA that writes a style sheet.

A style sheet is conformant to this specification if all of its statements that use syntax defined in this module are valid according to the generic CSS grammar and the individual grammars of each feature defined in this module.

A renderer is conformant to this specification if, in addition to interpreting the style sheet as defined by the appropriate specifications, it supports all the features defined by this specification by parsing them correctly and rendering the document accordingly. However, the inability of a UA to correctly render a document due to limitations of the device does not make the UA non-conformant. (For example, a UA is not required to render color on a monochrome monitor.)

An authoring tool is conformant to this specification if it writes style sheets that are syntactically correct according to the generic CSS grammar and the individual grammars of each feature in this module, and meet all other conformance requirements of style sheets as described in this module.

### Requirements for Responsible Implementation of CSS

The following sections define several conformance requirements for implementing CSS responsibly, in a way that promotes interoperability in the present and future.

#### Partial Implementations

So that authors can exploit the forward-compatible parsing rules to assign fallback values, CSS renderers must treat as invalid (and ignore as appropriate) any at-rules, properties, property values, keywords, and other syntactic constructs for which they have no usable level of support. In particular, user agents must not selectively ignore unsupported property values and honor supported values in a single multi-value property declaration: if any value is considered invalid (as unsupported values must be), CSS requires that the entire declaration be ignored.

#### Implementations of Unstable and Proprietary Features

To avoid clashes with future stable CSS features, the CSSWG recommends following best practices for the implementation of unstable features and proprietary extensions to CSS.

#### Implementations of CR-level Features

Once a specification reaches the Candidate Recommendation stage, implementers should release an unprefixed implementation of any CR-level feature they can demonstrate to be correctly implemented according to spec, and should avoid exposing a prefixed variant of that feature.

To establish and maintain the interoperability of CSS across implementations, the CSS Working Group requests that non-experimental CSS renderers submit an implementation report (and, if necessary, the testcases used for that implementation report) to the W3C before releasing an unprefixed implementation of any CSS features. Testcases submitted to W3C are subject to review and correction by the CSS Working Group.

Further information on submitting testcases and implementation reports can be found from on the CSS Working Group’s website at http://www.w3.org/Style/CSS/Test/. Questions should be directed to the public-css-testsuite@w3.org mailing list.

## Index

### Terms defined by this specification

- [[defaultElementStylesMap]], in §2
- flattened element tree, in §3.4
- flat tree, in §3.4
- :host, in §3.2.3
- :host(), in §3.2.3
- :host-context(), in §3.2.3
- in the context of a shadow tree, in §3.2.1
- in the context of the shadow tree, in §3.2.1
- ::slotted(), in §3.2.4
- tree context, in §3.2.1

https://drafts.csswg.org/css-cascade-4/#cascade Referenced in:

- 2. Default Styles for Custom Elements

https://drafts.csswg.org/css-cascade-4/#cascade-origin-ua Referenced in:

- 2. Default Styles for Custom Elements (2)
- 3.4.1. Slots and Slotted Elements in a Shadow Tree

https://drafts.csswg.org/css-cascade-4/#cascade-origin-ua Referenced in:

- 2. Default Styles for Custom Elements (2)
- 3.4.1. Slots and Slotted Elements in a Shadow Tree

https://drafts.csswg.org/css-display-3/#propdef-display Referenced in:

- 3.2.2. Selecting Shadow Hosts from within a Shadow Tree
- 3.4.1. Slots and Slotted Elements in a Shadow Tree (2)

https://drafts.csswg.org/css-fonts-4/#at-font-face-rule Referenced in:

- 2. Default Styles for Custom Elements

https://drafts.csswg.org/css-pseudo-4/#tree-abiding Referenced in:

- 3.2.4. Selecting Slot-Assigned Content: the ::slotted() pseudo-element

https://drafts.csswg.org/css-syntax-3/#at-rule Referenced in:

- 2. Default Styles for Custom Elements

https://dom.spec.whatwg.org/#concept-tree-child Referenced in:

- 3.2.1. Matching Selectors Against Shadow Trees
- 3.2.4. Selecting Slot-Assigned Content: the ::slotted() pseudo-element
- 3.4. Flattening the DOM into an Element Tree (2)

https://dom.spec.whatwg.org/#concept-tree-child Referenced in:

- 3.2.1. Matching Selectors Against Shadow Trees
- 3.2.4. Selecting Slot-Assigned Content: the ::slotted() pseudo-element
- 3.4. Flattening the DOM into an Element Tree (2)

https://dom.spec.whatwg.org/#concept-tree-descendant Referenced in:

- 3.1. Informative Explanation of Shadow DOM
- 3.2.1. Matching Selectors Against Shadow Trees

https://dom.spec.whatwg.org/#find-flattened-slotables Referenced in:

- 3.2.4. Selecting Slot-Assigned Content: the ::slotted() pseudo-element (2) (3)

https://dom.spec.whatwg.org/#find-slotables Referenced in:

- 3.4. Flattening the DOM into an Element Tree

https://dom.spec.whatwg.org/#concept-documentfragment-host Referenced in:

- 3.1. Informative Explanation of Shadow DOM

https://dom.spec.whatwg.org/#concept-light-tree Referenced in:

- 3.1. Informative Explanation of Shadow DOM (2)
- 3.2.1. Matching Selectors Against Shadow Trees
- 3.4. Flattening the DOM into an Element Tree (2)
- 3.4.1. Slots and Slotted Elements in a Shadow Tree

https://dom.spec.whatwg.org/#concept-element-local-name Referenced in:

- 2. Default Styles for Custom Elements (2)

https://dom.spec.whatwg.org/#dom-parentnode-queryselector Referenced in:

- 3.2.1. Matching Selectors Against Shadow Trees

https://dom.spec.whatwg.org/#concept-tree-root Referenced in:

- 3.2.1. Matching Selectors Against Shadow Trees
- 3.4. Flattening the DOM into an Element Tree

https://dom.spec.whatwg.org/#element-shadow-host Referenced in:

- 3.1. Informative Explanation of Shadow DOM (2) (3)
- 3.2.1. Matching Selectors Against Shadow Trees
- 3.2.2. Selecting Shadow Hosts from within a Shadow Tree (2) (3) (4) (5) (6) (7) (8) (9)
- 3.2.3. Selecting Into the Light: the :host, :host(), and :host-context() pseudo-classes (2) (3) (4) (5)
- 3.2.4. Selecting Slot-Assigned Content: the ::slotted() pseudo-element
- 3.4. Flattening the DOM into an Element Tree (2) (3)

https://dom.spec.whatwg.org/#concept-shadow-root Referenced in:

- 3.1. Informative Explanation of Shadow DOM (2)
- 3.2.1. Matching Selectors Against Shadow Trees (2) (3)
- 3.2.2. Selecting Shadow Hosts from within a Shadow Tree
- 3.4. Flattening the DOM into an Element Tree

https://dom.spec.whatwg.org/#concept-shadow-tree Referenced in:

- 2. Default Styles for Custom Elements (2) (3)
- 3.1. Informative Explanation of Shadow DOM (2) (3) (4) (5) (6) (7) (8)
- 3.2.1. Matching Selectors Against Shadow Trees (2) (3) (4)
- 3.2.2. Selecting Shadow Hosts from within a Shadow Tree (2) (3) (4) (5) (6) (7) (8)
- 3.2.3. Selecting Into the Light: the :host, :host(), and :host-context() pseudo-classes (2) (3) (4) (5) (6) (7)
- 3.2.4. Selecting Slot-Assigned Content: the ::slotted() pseudo-element
- 3.4. Flattening the DOM into an Element Tree (2) (3) (4)

https://dom.spec.whatwg.org/#concept-shadow-including-ancestor Referenced in:

- 3.2.3. Selecting Into the Light: the :host, :host(), and :host-context() pseudo-classes

https://dom.spec.whatwg.org/#concept-shadow-including-tree-order Referenced in:

- 3.3. Shadow Trees and the Cascade (2)

https://dom.spec.whatwg.org/#concept-slot Referenced in:

- 3.1. Informative Explanation of Shadow DOM (2) (3) (4) (5) (6) (7)
- 3.2.4. Selecting Slot-Assigned Content: the ::slotted() pseudo-element (2) (3) (4) (5) (6) (7) (8)
- 3.4. Flattening the DOM into an Element Tree (2) (3) (4)
- 3.4.1. Slots and Slotted Elements in a Shadow Tree (2) (3) (4) (5)

https://dom.spec.whatwg.org/#concept-slotable Referenced in:

- 3.4. Flattening the DOM into an Element Tree

https://html.spec.whatwg.org/multipage/window-object.html#window Referenced in:

- 2. Default Styles for Custom Elements

https://html.spec.whatwg.org/multipage/custom-elements.html#custom-element Referenced in:

- 2. Default Styles for Custom Elements

https://html.spec.whatwg.org/multipage/scripting.html#the-slot-element Referenced in:

- 3.1. Informative Explanation of Shadow DOM
- 3.2.4. Selecting Slot-Assigned Content: the ::slotted() pseudo-element (2) (3)

https://drafts.csswg.org/selectors-3/#x Referenced in:

- 3.2.4. Selecting Slot-Assigned Content: the ::slotted() pseudo-element

https://drafts.csswg.org/selectors-4/#selectordef-matches Referenced in:

- 3.2.3. Selecting Into the Light: the :host, :host(), and :host-context() pseudo-classes

https://drafts.csswg.org/selectors-4/#negation-pseudo Referenced in:

- 3.2.3. Selecting Into the Light: the :host, :host(), and :host-context() pseudo-classes

https://drafts.csswg.org/selectors-4/#typedef-compound-selector-list Referenced in:

- 3.2.3. Selecting Into the Light: the :host, :host(), and :host-context() pseudo-classes (2) (3)
- 3.2.4. Selecting Slot-Assigned Content: the ::slotted() pseudo-element (2)

https://drafts.csswg.org/selectors-4/#complex Referenced in:

- 2. Default Styles for Custom Elements

https://drafts.csswg.org/selectors-4/#compound Referenced in:

- 2. Default Styles for Custom Elements

https://drafts.csswg.org/selectors-4/#descendant-combinator Referenced in:

- 3.1. Informative Explanation of Shadow DOM

https://drafts.csswg.org/selectors-4/#featureless Referenced in:

- 3.2.2. Selecting Shadow Hosts from within a Shadow Tree

https://drafts.csswg.org/selectors-4/#match-a-selector-against-a-tree Referenced in:

- 3.2.1. Matching Selectors Against Shadow Trees

https://drafts.csswg.org/selectors-4/#match-a-selector-against-an-element Referenced in:

- 3.2.4. Selecting Slot-Assigned Content: the ::slotted() pseudo-element

https://drafts.csswg.org/selectors-4/#specificity Referenced in:

- 3.2.3. Selecting Into the Light: the :host, :host(), and :host-context() pseudo-classes (2) (3) (4) (5)
- 3.2.4. Selecting Slot-Assigned Content: the ::slotted() pseudo-element (2)

https://drafts.csswg.org/selectors-4/#type-selector Referenced in:

- 2. Default Styles for Custom Elements (2)

### Terms defined by reference

- [css-cascade-4] defines the following terms:
  - cascade
  - ua origin
  - user agent origin
- [css-display-3] defines the following terms:
  - display
- [css-fonts-4] defines the following terms:
  - @font-face
- [css-pseudo-4] defines the following terms:
  - tree-abiding pseudo-element
- [css-syntax-3] defines the following terms:
  - at-rule
- [DOM] defines the following terms:
  - child
  - children
  - descendant
  - find flattened slotables
  - find slotables
  - host
  - light tree
  - local name
  - querySelector(selectors)
  - root
  - shadow host
  - shadow root
  - shadow tree
  - shadow-including ancestor
  - shadow-including tree order
  - slot
  - slotable
- [HTML] defines the following terms:
  - Window
  - custom element
  - slot
- [selectors-3] defines the following terms:
  - *
- [selectors-4] defines the following terms:
  - :matches()
  - :not()
  - <compound-selector-list>
  - complex selector
  - compound selector
  - descendant combinator
  - featureless
  - match a selector against a tree
  - match a selector against an element
  - specificity
  - type selector

## References

### Normative References

[CSS-CASCADE-4]
  Elika Etemad; Tab Atkins Jr.. CSS Cascading and Inheritance Level 4. 28 August 2018. CR. URL: https://www.w3.org/TR/css-cascade-4/
[CSS-DISPLAY-3]
  Tab Atkins Jr.; Elika Etemad. CSS Display Module Level 3. 28 August 2018. CR. URL: https://www.w3.org/TR/css-display-3/
[CSS-FONTS-4]
  John Daggett; Myles Maxfield; Chris Lilley. CSS Fonts Module Level 4. 20 September 2018. WD. URL: https://www.w3.org/TR/css-fonts-4/
[CSS-PSEUDO-4]
  Daniel Glazman; Elika Etemad; Alan Stearns. CSS Pseudo-Elements Module Level 4. 25 February 2019. WD. URL: https://www.w3.org/TR/css-pseudo-4/
[CSS-SYNTAX-3]
  Tab Atkins Jr.; Simon Sapin. CSS Syntax Module Level 3. 20 February 2014. CR. URL: https://www.w3.org/TR/css-syntax-3/
[CSS3CASCADE]
  Elika Etemad; Tab Atkins Jr.. CSS Cascading and Inheritance Level 3. 28 August 2018. CR. URL: https://www.w3.org/TR/css-cascade-3/
[DOM]
  Anne van Kesteren. DOM Standard. Living Standard. URL: https://dom.spec.whatwg.org/
[HTML]
  Anne van Kesteren; et al. HTML Standard. Living Standard. URL: https://html.spec.whatwg.org/multipage/
[RFC2119]
  S. Bradner. Key words for use in RFCs to Indicate Requirement Levels. March 1997. Best Current Practice. URL: https://tools.ietf.org/html/rfc2119
[SELECTORS-4]
  Elika Etemad; Tab Atkins Jr.. Selectors Level 4. 21 November 2018. WD. URL: https://www.w3.org/TR/selectors-4/

### Informative References

[SELECTORS-3]
  Tantek Çelik; et al. Selectors Level 3. 6 November 2018. REC. URL: https://www.w3.org/TR/selectors-3/

## Issues Index

Do we need to restrict the at-rules that can be used in these sheets? For example, do we allow an @font-face? I’m going to leave it as allowed unless/until I hear complaints. ↵
A non-obvious result of this is that elements assigned to a slot inherit from that slot, not their light-tree parent or any deeper slots their slot gets assigned to. This means that text nodes are styled by the shadow tree of their parent, with nobody else capable of intervening in any way. Do we want an additional pseudo-element for targeting those text nodes so they can be styled at all slot-assignment levels, like normal elements can be? This implies it needs to work for text nodes in the light tree before they’re assigned downwards, so this can’t just be a ::slotted() variant. Luckily, this is a long-standing request! ↵
#dom-window-defaultelementstylesmap-slot Referenced in:

- 2. Default Styles for Custom Elements

#tree-context Referenced in:

- 3.2.1. Matching Selectors Against Shadow Trees (2)
- 3.3. Shadow Trees and the Cascade

#in-the-context-of-a-shadow-tree Referenced in:

- 3.2.1. Matching Selectors Against Shadow Trees
- 3.2.2. Selecting Shadow Hosts from within a Shadow Tree
- 3.2.3. Selecting Into the Light: the :host, :host(), and :host-context() pseudo-classes (2) (3)

#selectordef-host Referenced in:

- 3.2.2. Selecting Shadow Hosts from within a Shadow Tree (2)
- 3.2.3. Selecting Into the Light: the :host, :host(), and :host-context() pseudo-classes (2) (3) (4) (5)

#selectordef-host-function Referenced in:

- 3.2.2. Selecting Shadow Hosts from within a Shadow Tree
- 3.2.3. Selecting Into the Light: the :host, :host(), and :host-context() pseudo-classes (2)

#selectordef-host-context Referenced in:

- 3.2.2. Selecting Shadow Hosts from within a Shadow Tree
- 3.2.3. Selecting Into the Light: the :host, :host(), and :host-context() pseudo-classes (2) (3)

#selectordef-slotted Referenced in:

- 3.2.4. Selecting Slot-Assigned Content: the ::slotted() pseudo-element (2) (3) (4) (5) (6) (7) (8) (9) (10)
- 3.4. Flattening the DOM into an Element Tree

#flat-tree Referenced in:

- 3.1. Informative Explanation of Shadow DOM
- 3.4. Flattening the DOM into an Element Tree (2) (3) (4)